/*
 * Sonar, entreprise quality control tool.
 * Copyright (C) 2007-2008 Hortis-GRC SA
 * mailto:be_agile HAT hortis DOT ch
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with Sonar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package org.sonar.maven.pmd;

import ch.hortis.sonar.model.Rule;
import ch.hortis.sonar.model.RuleFailureLevel;
import ch.hortis.sonar.model.RuleFailureParam;
import ch.hortis.sonar.mvn.PomUtils;
import org.apache.maven.project.MavenProject;
import org.sonar.commons.RulesRepository;
import org.sonar.maven.AbstractCollector;
import org.sonar.maven.MeasuresRecorder;
import org.sonar.maven.XmlReportParser;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import java.io.File;
import java.util.List;

public class PmdCollector extends AbstractCollector {
  private XmlReportParser parser;
  private MeasuresRecorder recorder;
  private RulesRepository rulesRepository;

  public PmdCollector(MavenProject project, MeasuresRecorder recorder, RulesRepository rulesRepository) {
    File reportFile = findFileFromBuildDirectory(project, "pmd.xml");
    if (reportFile != null) {
      parser = new XmlReportParser();
      parser.parse(reportFile);
      this.recorder = recorder;
      this.rulesRepository = rulesRepository;
    }
  }

  public PmdCollector(File xmlFile, MeasuresRecorder recorder, RulesRepository rulesRepository) {
    parser = new XmlReportParser();
    parser.parse(xmlFile);
    this.recorder = recorder;
    this.rulesRepository = rulesRepository;
  }

  public void collect() {
    if (parser != null) {
      NodeList files = parser.executeXPathNodeList("/pmd/file");
      for (int i = 0; i < files.getLength(); i++) {
        Element element = (Element) files.item(i);
        List<Element> violations = parser.getChildElements(element, "violation");
        for (Element violation : violations) {
          Rule rule = rulesRepository.getRuleByPluginKey(violation.getAttribute("rule"));
          if (rule == null) {
            continue;
          }

          String message = violation.getFirstChild().getNodeValue();
          RuleFailureLevel level = RuleFailureLevel.INFO;
          int priority = Integer.parseInt(violation.getAttribute("priority"));
          if (priority <= 2) {
            level = RuleFailureLevel.ERROR;
          } else if (priority == 3 || priority == 4) {
            level = RuleFailureLevel.WARNING;
          }

          RuleFailureParam lineParam = null, columnParam = null;
          String line = violation.getAttribute("beginline");
          if (line != null && !"".equals(line)) {
            lineParam = new RuleFailureParam("line", Double.valueOf(line), null);
          }
          String column = violation.getAttribute("begincolumn");
          if (column != null && !"".equals(column)) {
            columnParam = new RuleFailureParam("column", Double.valueOf(column), null);
          }
          recorder.createClassRuleFailure(rule, violation.getAttribute("package"), PomUtils.extractClassName(element), false, message, level, lineParam, columnParam);
        }
      }
    }
  }

}
